<?php
/*
 * exercise.php - base class for all exercise types
 * Copyright (c) 2003-2005  Erik Schmitt, Philipp Hgelmeyer
 * Copyright (c) 2005-2006  Christa Deiwiks
 * Copyright (c) 2006-2008  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

/**
 * This class provides methods accessible by exercise-classes implementing a
 * unique type of exercise. The methods to be used by inheriting exercise
 * classes invoke internal methods to parse XML and return values encapsulated
 * by XML.
 *
 * @abstract
 * @author Erik Schmitt
 * @date 2003-02-25
 *
 * @todo inherition of class variables works not properly (in php). #cdeiwiks ???
 */

class Exercise
{
    var $type,
        $name,
        $question,
        $hint,
        $commentAllowed,
        $numberOfAnswers,
        $answerArray = array(),
        $correctArray = array(),
        $evaluationMode,
        $language = 'de';

    // constructor
    function Exercise($xml = null)
    {
        if (!isset($xml)) {  // construct exercise using request parameters
            $this->name            = vips_get_string_param('exercise_name', true);
            $this->question        = vips_get_string_param('exercise_question', true);
            $this->hint            = vips_get_string_param('exercise_hint', true);
            $this->commentAllowed  = vips_get_int_param('exercise_comment');
            $this->numberOfAnswers = vips_get_int_param('number_of_answers');
        }
        else {  // construct exercise using XML
            $dom_xml = $this->getDomXML($xml);

            $name_node     = reset($dom_xml->get_elements_by_tagname('Name'));
            $question_node = reset($dom_xml->get_elements_by_tagname('Question'));
            $hint_node     = reset($dom_xml->get_elements_by_tagname('Hint'));
            $comment_node  = reset($dom_xml->get_elements_by_tagname('CommentField'));

            $this->name           = is_object($name_node) ? utf8_decode($name_node->get_content()) : null;
            $this->question       = is_object($question_node) ? utf8_decode($question_node->get_content()) : null;
            $this->hint           = is_object($hint_node) ? utf8_decode($hint_node->get_content()) : null;
            $this->commentAllowed = is_object($comment_node) ? $comment_node->get_attribute('Visible') : 0;
        }
    }

    function getType()
    {
        return $this->type;
    }

    function getName()
    {
        return $this->name;
    }

    function getQuestion()
    {
        return $this->question;
    }

    function getHint()
    {
        return $this->hint;
    }

    function getCommentAllowed()
    {
        return $this->commentAllowed;
    }

    function getNumberOfAnswers()
    {
        return $this->numberOfAnswers;
    }

    function getAnswerArray()
    {
        return $this->answerArray;
    }

    function getCorrectArray()
    {
        return $this->correctArray;
    }

    function getEvaluationMode()
    {
        return $this->evaluationMode;
    }

    function setEvaluationMode($evaluation_mode)
    {
        $this->evaluationMode = $evaluation_mode;
    }

    /**
     * Should be overwritten by every exercise type which has to pass some
     * additional information (i.e. everything besides type, name, question,
     * hint, comment allowed, number of answers, answer array and correct
     * array).
     */
    function getQuirks()
    {
        return null;
    }

    function getQuirk($name)
    {
        $quirks = $this->getQuirks();
        return $quirks[$name];
    }

    function getMaxPoints()
    {
        return $this->numberOfAnswers;
    }
    
    /**
     * Should be overwritten by any exercise type which would not work properly
     * without some default values set to sensible values.
     */
    function setDefaultValues()
    {
    }



   /**
    *    XXX
    *
    *    @abstract
    *    @access public
    *    @returns array with int (from 0 to 100 ) as an % value and save t/f
    */

    function evaluate($exercise_id,$solution_id){
    }


  /**
   * Generates the HTML-Output of an exercise to represent
   * it to the tutor to coorect a solution.
   *
   * @param Number exercise_id the unique id of the exercise
   * @param XML-String $p_exerciseXML the definition of the exercise in XML
   * @param XML-String $p_solutionXML the former solution of the learner. Optional.
   */


        function genCorrection
                        ( $p_exercise_id
                        , $p_exercise_sheet_id
                        , $p_exerciseXML
                        , $p_exercise_name
                        , $p_solutionXML = ""
                        , $p_exercise_solution_id = ""
                        , $p_student_comment
                        , $p_solution
                        )
        {
                $msg = $this->printFormToGetAnswer   // genOutput
                        ( $p_exercise_id
                        , $p_exercise_sheet_id
                        , $p_exerciseXML
                        , $p_exercise_name
                        , $p_solutionXML
                        , $p_exercise_solution_id
                        , $p_student_comment
                        , $p_solution
                        );

                # Punkte anzeigen und ndern knnen.
                return $msg;
        }

        /**
    *   Returns on solution to an exercise
    *
    *    @abstract
    *    @access public
    *    @returns string
    */
    function getSolution($exercise_id,$solution_id){
    }

   /**
    *    Returns solution to an Exercise
    *
    *    @abstract
    *    @access public
    *    @returns string
    */

    function genSolution($exercise_id){
    }


    /**
    *   marks a solution as the one to select
    *
    *    @abstract
    *    @access public
    *    @returns string
    */

    function setSolution($exercise_id,$solution_id){
    }

      /**
    *  returns an statistic output.
    *
    *    @abstract
    *    @access public
    *    @returns string
    */
    function getStatistic($exercise_id,$course_id){
    }


        /**************************************************************************/
        /*
        /* XML-related functions
        /*
        /**************************************************************************/



        /**
        * Returns the dom-document object of a given XML-String.
        *
        * @author Erik Schmitt
        * @param String $p_xml_string the xml-data as a string
        * @return Object the dom-document object
        */

        function getDomXML($p_xml_string)
        {
                // suppress warning, error message and dump is given out
                // below in case of errors.
                $dom_exerciseXML = domxml_open_mem(utf8_encode($p_xml_string));

                // Attention! CDATA sections get lost with calling domxml_open_mem()!

                if ($dom_exerciseXML === FALSE)
                {
                        echo _("<hr />Error while parsing the document. Standard error message suppressed. Dump follows. Exit system.");
                        echo '<p />' . _("DUMP: ") . html_ready($p_xml_string);
                        echo '<p />' . _("Exit.") . '<hr />';
                        exit;
                }
                return ($dom_exerciseXML);
        }



        /**
         * Returns the content of all tags looked up in an xml-string as an
         * array.
         *
         * Returns empty array if nothing was found.
         *
         * @author Erik Schmitt
         * @date 2003-02-23
         * @param String $p_xml_string the xml-data as a string
         * @param String $p_lookup_tag the tag to look up in the xml-string
         * @return Array $m_return_value the array with the content found
         */

        function getTagsFromXML($p_xml_string, $p_lookup_tag) {

                $m_return_value = array();

                if($p_xml_string != "") { // is empty when a new exercise is created

                        $m_exercise_dom_xml = $this->getDomXML($p_xml_string);

                        $m_exercise_elements = $m_exercise_dom_xml->get_elements_by_tagname($p_lookup_tag);

                        $i = 0;
                        foreach ($m_exercise_elements as $knod)
                        {
                                $m_return_value[$i++] = utf8_decode($knod->get_content());
                        }
                }
                return ($m_return_value);
        }



        /**
         * Returns the content of all tags looked up in an xml-string as an
         * array.
         *
         * Returns empty array if nothing was found.
         *
         * @author Philipp Hgelmeyer
         * @date 2004-06-29
         * @param String $p_xml_string the xml-data as a string
         * @param String $p_lookup_tag the tag to look up in the xml-string
         * @param String $p_attribute the attribute to look up in each tag
         * @return Array $m_return_value the array with the content found
         */

         function getAttributesFromXML($p_xml_string, $p_lookup_tag,$p_attribute) {

        $m_return_value = array();

        if($p_xml_string != "") { // is empty when a new exercise is created
                        $m_exercise_dom_xml = $this->getDomXML($p_xml_string);
            $m_exercise_elements = $m_exercise_dom_xml->get_elements_by_tagname($p_lookup_tag);
            $i = 0;
            foreach ($m_exercise_elements as $knod)
            {
                $m_return_value[$i++] = utf8_decode($knod->get_attribute($p_attribute));
            }
        }
        return ($m_return_value);
    }

    /**
     * Export this exercise to plain text format.
     */
    function exportText ($exercise_tag = NULL)
    {
        if ($exercise_tag == NULL)
        {
            return '# Aufgaben des Typs "'.get_class($this).'" knnen nicht exportiert werden.'."\n";
        }

        $result = 'Name: '.$this->name."\n";
        $result .= $exercise_tag.': '.$this->question."\n";

        if ($this->hint != '')
        {
            $result .= "Tipp:\n";
            $result .= $this->hint."\n";
            $result .= "\\Tipp\n";
        }

        return $result;
    }
} // end class
?>
