<?php
/*
 *  $Id: TableInfo.php,v 1.6 2005/03/01 13:56:43 micha Exp $
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many individuals
 * and is licensed under the LGPL. For more information please see
 * <http://creole.phpdb.org>.
 */

/**
 * Represents a table.
 *
 * @author    Hans Lellelid <hans@xmpl.org>
 * @author    Michael Aichler <aichler@mediacluster.de>
 * @version   $Revision: 1.6 $
 * @package   creole.metadata
 */
class TableInfo
{
  /**
  * Name of the table.
  * @var string
  */
  var $name;
  /**
  * Array of columns in this table.
  * @var array
  */
  var $columns = array();
  /**
  * Array of foreign keys in this table.
  * @var array
  */
  var $foreignKeys = array();
  /**
  * Array of indexes in this table.
  * @var array
  */
  var $indexes = array();
  /**
  * Array of foreign keys in this table.
  * @var array
  */
  var $primaryKey;
  /**
  * Specifies whether primary keys have been loaded.
  * @var bool
  */
  var $pkLoaded = false;
  /**
  * Specifies whether foreign keys have been loaded.
  * @var bool
  */
  var $fksLoaded = false;
  /**
  * Specifies whether indexes have been loaded.
  * @var bool
  */
  var $indexesLoaded = false;
  /**
  * Specifies whether columns have been loaded.
  * @var bool
  */
  var $colsLoaded = false;
  /**
  * Specifies whether vendor information have been loaded.
  * @var bool
  */
  var $vendorLoaded = false;
  /**
  * Additional and optional vendor specific information.
  * @var array
  */
  var $vendorSpecificInfo = array();
  /**
  * Database Connection.
  * @var Connection
  */
  var $conn;
  /**
  * The parent DatabaseInfo object.
  * @var DatabaseInfo
  */
  var $database;
  /**
  * Shortcut to db resource link id (needed by drivers for queries).
  * @var Resource
  */
  var $dblink;
  /**
  * Shortcut to db name (needed by many drivers for queries).
  * @var string
  */
  var $dbname;


  /**
  * TableInfo constructor.
  *
  * @param string $table The table name.
  * @param string $database The database name.
  * @param resource $dblink The db connection resource.
  */
  function TableInfo(/*DatabaseInfo*/ &$database, $name) 
  {
    Creole::typeHint($database, 'DatabaseInfo', 'TableInfo', 'TableInfo');

    $this->database =& $database;
    $this->name = $name;
    $this->conn =& $database->getConnection(); // shortcut because all drivers need this for the queries
    $this->dblink =& $this->conn->getResource();
    $this->dbname = $database->getName();
  }

  /**
  * This "magic" method is invoked upon serialize().
  * Because the Info class hierarchy is recursive, we must handle
  * the serialization and unserialization of this object.
  *
  * @return array The class variables that should be serialized (all must be public!).
  */
  function __sleep()
  {
    return array('name', 'columns', 'foreignKeys', 'indexes', 'primaryKey');
  }

  /**
  * This "magic" method is invoked upon unserialize().
  * This method re-hydrates the object and restores the recursive hierarchy.
  */
  function __wakeup()
  {
    // restore chaining
    for($i=0,$j=count($this->columns); $i < $j; $i++) {
      $col =& $this->columns[$i];
      $col->table =& $this;
    }
  }

  /**
  * Get table name.
  *
  * @return string
  */
  function getName()
  {
    return $this->name;
  }

  /**
  * Get the parent DatabaseInfo object.
  *
  * @return DatabaseInfo
  */
  function & getDatabase()
  {
    return $this->database;
  }

  /**
  * Get parimary key in this table.
  * @return mixed array ForeignKeyInfo[] on success, SQLException - if foreign keys are unsupported by DB.
  */
  function & getPrimaryKey()
  {
    if(!$this->pkLoaded) {
      if (($e = $this->initPrimaryKey()) !== true) {
        return $e;
      }
    }
    return $this->primaryKey;
  }

  /**
  * Get array of columns for this table.
  *
  * @return mixed array ColumnInfo[] on success, SQLException on failure.
  */
  function & getColumns()
  {
    if(!$this->colsLoaded) {
      if (($e = $this->initColumns()) !== true) {
        return $e;
      }
    }
    return array_values($this->columns); // re-key numerically
  }

  /**
  * Get the ColumnInfo object for specified column.
  *
  * @param string $name The column name.
  * @return mixed ColumnInfo on success, SQLException if column does not exist for this table.
  */
  function & getColumn($name)
  {
    if(!$this->colsLoaded) {
      if (($e = $this->initColumns()) !== true) {
        return $e;
      }
    }
    
    if (!isset($this->columns[$name])) {
      return new SQLException(CREOLE_ERROR_NOSUCHFIELD, "Table `".$this->name."` has no column `".$name."`");
    }
    
    return $this->columns[$name];
  }

  /**
  * Returns whether a column with name <em>$name</em> exists.
  *
  * @param string $name
  * @return bool
  */
  function hasColumn($name)
  {
    return isset($this->columns[$name]);
  }

  /**
  * Adds a column to this table.
  *
  * @param ColumnInfo $column
  * @return void
  */
  function addColumn(&$column)
  {
    Creole::typeHint($column, 'ColumnInfo', 'TableInfo', 'addColumn');
    $this->columns[$column->getName()] =& $column;
  }

  /**
  * Get all foreign keys.
  *
  * @return mixed array ForeignKeyInfo[] on success, SQLException on failure.
  */
  function & getForeignKeys()
  {
    if(!$this->fksLoaded) {
      if (($e = $this->initForeignKeys()) !== true) {
        return $e;
      }
    }
    
    return array_values($this->foreignKeys);
  }

  /**
  * Get specified fk for this table.
  *
  * @param string $name The foreign key name to retrieve.
  * @return mixed ForeignKeyInfo on success or SQLException - if fkey does not exist for this table.
  */
  function & getForeignKey($name)
  {
    if(!$this->fksLoaded) {
      if (($e = $this->initForeignKeys()) !== true) {
        return $e;
      }
    }
    if (!isset($this->foreignKeys[$name])) {
      return new SQLException(CREOLE_ERROR_NOSUCHFIELD, "Table `".$this->name."` has no foreign key `".$name."`");
    }
    return $this->foreignKeys[$name];
  }

  /**
  * Get array of IndexInfo objects for this table.
  *
  * @return mixed array IndexInfo[] on success, SQLException on failure.
  */
  function & getIndexes()
  {
    if(!$this->indexesLoaded) {
      if (($e = $this->initIndexes()) !== true) {
        return $e;
      }
    }

    return array_values($this->indexes);
  }

  /**
  * Gets the IndexInfo object for a specified index.
  *
  * @param string $name The index name to retrieve.
  * @return mixed IndexInfo on success, SQLException - if index does not exist for this table.
  */
  function & getIndex($name)
  {
    if(! $this->indexesLoaded) {
      if (($e = $this->initIndexes()) !== true) {
        return $e;
      }
    }

    if (!isset($this->indexes[$name])) {
      return new SQLException(CREOLE_ERROR_NOSUCHFIELD, "Table `".$this->name."` has no index `".$name."`");
    }

    return $this->indexes[$name];
  }

  /**
  * Alias for getIndexes() method.
  * @return array
  */
  function & getIndices()
  {
    return $this->getIndexes();
  }

  /**
  * @return string
  */
  function toString()
  {
    return $this->name;
  }

  /**
  * Have foreign keys been loaded ?
  *
  * @return bool
  */
  function foreignKeysLoaded()
  {
    return $this->fksLoaded;
  }

  /**
  * Has primary key info been loaded ?
  *
  * @return bool
  */
  function primaryKeyLoaded()
  {
    return $this->pkLoaded;
  }

  /**
  * Have columns been loaded?
  *
  * @return bool
  */
  function columnsLoaded()
  {
    return $this->colsLoaded;
  }

  /**
  * Has index information been loaded ?
  *
  * @return bool
  */
  function indexesLoaded()
  {
    return $this->indexesLoaded;
  }

  /**
  * Has vendor specific information been loaded ?
  *
  * @return bool
  */
  function vendorSpecificInfoLoaded()
  {
    return $this->vendorLoaded;
  }

  /**
  * Loads the columns.
  * @return void
  */
  function initColumns()
  {
    trigger_error (
      "TableInfo::initColumns(): abstract function has to be reimplemented !",
      E_USER_ERROR
    );
  }

  /**
  * Loads the primary key information for this table.
  * @return void
  */
  function initPrimaryKey()
  {
    trigger_error (
      "TableInfo::initPrimaryKey(): abstract function has to be reimplemented !",
      E_USER_ERROR
    );
  }

  /**
  * Loads the foreign keys for this table.
  * @return void
  */
  function initForeignKeys()
  {
    trigger_error (
      "TableInfo::initForeignKeys(): abstract function has to be reimplemented !",
      E_USER_ERROR
    );
  }

  /**
  * Loads the indexes information for this table.
  * @return void
  */
  function initIndexes()
  {
    trigger_error (
      "TableInfo::initIndexes(): abstract function has to be reimplemented !",
      E_USER_ERROR
    );
  }

  /**
  * Loads the vendor specific information for this table.
  *
  * @return void
  */
  function initVendorSpecificInfo()
  {
    /*
    //it must be asbtract and be implemented in every vendor specific driver,
    //however since it's an experimental stuff it has an empty body in order
    //not to break BC

    trigger_error (
      "TableInfo::(): abstract function has to be reimplemented !",
      E_USER_ERROR
    );
    */
  }

}
